<h3>Installation on Mac OS X</h3>
<p>Copy <code>{{name}}.keylayout</code> and <code>{{name}}.icns</code> to either<br><code>~/Library/Keyboard Layouts/</code> or <code>/Library/Keyboard Layouts/</code>.</p>
<p>Open System Preferences, search for "keyboard layout,"<br>and add the {{name}} keyboard layout or input method.</p>
<p>Select {{name}} from the keyboard menu and start typing.</p>
<h3>Installation on Windows</h3>
<p>From the <code>Windows</code> directory, run <code>setup.exe</code> and follow the prompts.</p>
<p>Select {{name}} from the language bar and start typing.</p>
<h3>Installation on Windows (Alternative Method)</h3>
<p>Download and install <a href="https://keyman.com/windows/download#standalone" target="_blank">Keyman Desktop</a>.</p>
<p>From the <code>Keyman</code> directory, open and install the <code>{{name}}.kmp</code> file.</p>
<p>Select {{name}} from the Keyman system tray menu and start typing.</p>
<h3>Installation on Linux</h3>
<p>Open a terminal in the <code>Linux</code> directory and run <code>sudo python install.py</code>.</p>
<p>Restart the window server by logging out and in again.</p>
<p>Open the settings application, search for "keyboard layout,"<br>and add the {{name}} keyboard layout or input method.</p>
<p>Select {{name}} from the keyboard menu and start typing.</p>
<h3>Enjoy!</h3>
