package com.kreative.bitsnpicas.truetype;

public interface PlatformConstants {
	public static final int PLATFORM_ID_UNICODE = 0;
	public static final int PLATFORM_ID_MACINTOSH = 1;
	public static final int PLATFORM_ID_ISO_10646 = 2;
	public static final int PLATFORM_ID_WINDOWS = 3;
	public static final int PLATFORM_ID_WINDOWS_UNICODE = 10;
	
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_DEFAULT = 0;
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_1_1 = 1;
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_ISO_10646 = 2;
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_2_0 = 3;
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_2_0_NON_BMP = 4;
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_VARIATION_SEQUENCES = 5;
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_FULL = 6;
	public static final int PLATFORM_SPECIFIC_ID_UNICODE_FONTFORGE = 10;
	
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_ROMAN = 0;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_JAPANESE = 1;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_TRADITIONAL_CHINESE = 2;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_KOREAN = 3;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_ARABIC = 4;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_HEBREW = 5;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_GREEK = 6;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_RUSSIAN = 7;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_SYMBOL = 8;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_DEVANAGARI = 9;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_GURMUKHI = 10;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_GUJARATI = 11;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_ORIYA = 12;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_BENGALI = 13;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_TAMIL = 14;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_TELUGU = 15;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_KANNADA = 16;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_MALAYALAM = 17;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_SINHALESE = 18;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_BURMESE = 19;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_KHMER = 20;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_THAI = 21;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_LAOTIAN = 22;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_GEORGIAN = 23;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_ARMENIAN = 24;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_SIMPLIFIED_CHINESE = 25;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_TIBETAN = 26;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_MONGOLIAN = 27;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_GEEZ = 28;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_SLAVIC = 29;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_VIETNAMESE = 30;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_SINDHI = 31;
	public static final int PLATFORM_SPECIFIC_ID_MACINTOSH_UNINTERPRETED = 32;
	
	public static final int PLATFORM_SPECIFIC_ID_WINDOWS_SYMBOL = 0;
	public static final int PLATFORM_SPECIFIC_ID_WINDOWS_UNICODE_16 = 1;
	public static final int PLATFORM_SPECIFIC_ID_WINDOWS_SHIFT_JIS = 2;
	public static final int PLATFORM_SPECIFIC_ID_WINDOWS_PRC = 3;
	public static final int PLATFORM_SPECIFIC_ID_WINDOWS_BIG5 = 4;
	public static final int PLATFORM_SPECIFIC_ID_WINDOWS_JOHAB = 5;
	public static final int PLATFORM_SPECIFIC_ID_WINDOWS_UNICODE_32 = 10;
	
	public static final int LANGUAGE_ID_MACINTOSH_ENGLISH = 0;
	public static final int LANGUAGE_ID_MACINTOSH_FRENCH = 1;
	public static final int LANGUAGE_ID_MACINTOSH_GERMAN = 2;
	public static final int LANGUAGE_ID_MACINTOSH_ITALIAN = 3;
	public static final int LANGUAGE_ID_MACINTOSH_DUTCH = 4;
	public static final int LANGUAGE_ID_MACINTOSH_SWEDISH = 5;
	public static final int LANGUAGE_ID_MACINTOSH_SPANISH = 6;
	public static final int LANGUAGE_ID_MACINTOSH_DANISH = 7;
	public static final int LANGUAGE_ID_MACINTOSH_PORTUGUESE = 8;
	public static final int LANGUAGE_ID_MACINTOSH_NORWEGIAN = 9;
	public static final int LANGUAGE_ID_MACINTOSH_HEBREW = 10;
	public static final int LANGUAGE_ID_MACINTOSH_JAPANESE = 11;
	public static final int LANGUAGE_ID_MACINTOSH_ARABIC = 12;
	public static final int LANGUAGE_ID_MACINTOSH_FINNISH = 13;
	public static final int LANGUAGE_ID_MACINTOSH_GREEK = 14;
	public static final int LANGUAGE_ID_MACINTOSH_ICELANDIC = 15;
	public static final int LANGUAGE_ID_MACINTOSH_MALTESE = 16;
	public static final int LANGUAGE_ID_MACINTOSH_TURKISH = 17;
	public static final int LANGUAGE_ID_MACINTOSH_CROATIAN = 18;
	public static final int LANGUAGE_ID_MACINTOSH_TRADITIONAL_CHINESE = 19;
	public static final int LANGUAGE_ID_MACINTOSH_URDU = 20;
	public static final int LANGUAGE_ID_MACINTOSH_HINDI = 21;
	public static final int LANGUAGE_ID_MACINTOSH_THAI = 22;
	public static final int LANGUAGE_ID_MACINTOSH_KOREAN = 23;
	public static final int LANGUAGE_ID_MACINTOSH_LITHUANIAN = 24;
	public static final int LANGUAGE_ID_MACINTOSH_POLISH = 25;
	public static final int LANGUAGE_ID_MACINTOSH_HUNGARIAN = 26;
	public static final int LANGUAGE_ID_MACINTOSH_ESTONIAN = 27;
	public static final int LANGUAGE_ID_MACINTOSH_LATVIAN = 28;
	public static final int LANGUAGE_ID_MACINTOSH_SAMI = 29;
	public static final int LANGUAGE_ID_MACINTOSH_FAROESE = 30;
	public static final int LANGUAGE_ID_MACINTOSH_FARSI = 31;
	public static final int LANGUAGE_ID_MACINTOSH_RUSSIAN = 32;
	public static final int LANGUAGE_ID_MACINTOSH_SIMPLIFIED_CHINESE = 33;
	public static final int LANGUAGE_ID_MACINTOSH_FLEMISH = 34;
	public static final int LANGUAGE_ID_MACINTOSH_IRISH_GAELIC = 35;
	public static final int LANGUAGE_ID_MACINTOSH_ALBANIAN = 36;
	public static final int LANGUAGE_ID_MACINTOSH_ROMANIAN = 37;
	public static final int LANGUAGE_ID_MACINTOSH_CZECH = 38;
	public static final int LANGUAGE_ID_MACINTOSH_SLOVAK = 39;
	public static final int LANGUAGE_ID_MACINTOSH_SLOVENIAN = 40;
	public static final int LANGUAGE_ID_MACINTOSH_YIDDISH = 41;
	public static final int LANGUAGE_ID_MACINTOSH_SERBIAN = 42;
	public static final int LANGUAGE_ID_MACINTOSH_MACEDONIAN = 43;
	public static final int LANGUAGE_ID_MACINTOSH_BULGARIAN = 44;
	public static final int LANGUAGE_ID_MACINTOSH_UKRAINIAN = 45;
	public static final int LANGUAGE_ID_MACINTOSH_BYELORUSSIAN = 46;
	public static final int LANGUAGE_ID_MACINTOSH_UZBEK = 47;
	public static final int LANGUAGE_ID_MACINTOSH_KAZAKH = 48;
	public static final int LANGUAGE_ID_MACINTOSH_AZERBAIJANI_CYRILLIC = 49;
	public static final int LANGUAGE_ID_MACINTOSH_AZERBAIJANI_ARABIC = 50;
	public static final int LANGUAGE_ID_MACINTOSH_ARMENIAN = 51;
	public static final int LANGUAGE_ID_MACINTOSH_GEORGIAN = 52;
	public static final int LANGUAGE_ID_MACINTOSH_MOLDAVIAN = 53;
	public static final int LANGUAGE_ID_MACINTOSH_KIRGHIZ = 54;
	public static final int LANGUAGE_ID_MACINTOSH_TAJIKI = 55;
	public static final int LANGUAGE_ID_MACINTOSH_TURKMEN = 56;
	public static final int LANGUAGE_ID_MACINTOSH_MONGOLIAN_MONGOLIAN = 57;
	public static final int LANGUAGE_ID_MACINTOSH_MONGOLIAN_CYRILLIC = 58;
	public static final int LANGUAGE_ID_MACINTOSH_PASHTO = 59;
	public static final int LANGUAGE_ID_MACINTOSH_KURDISH = 60;
	public static final int LANGUAGE_ID_MACINTOSH_KASHMIRI = 61;
	public static final int LANGUAGE_ID_MACINTOSH_SINDHI = 62;
	public static final int LANGUAGE_ID_MACINTOSH_TIBETAN = 63;
	public static final int LANGUAGE_ID_MACINTOSH_NEPALI = 64;
	public static final int LANGUAGE_ID_MACINTOSH_SANSKRIT = 65;
	public static final int LANGUAGE_ID_MACINTOSH_MARATHI = 66;
	public static final int LANGUAGE_ID_MACINTOSH_BENGALI = 67;
	public static final int LANGUAGE_ID_MACINTOSH_ASSAMESE = 68;
	public static final int LANGUAGE_ID_MACINTOSH_GUJARATI = 69;
	public static final int LANGUAGE_ID_MACINTOSH_PUNJABI = 70;
	public static final int LANGUAGE_ID_MACINTOSH_ORIYA = 71;
	public static final int LANGUAGE_ID_MACINTOSH_MALAYALAM = 72;
	public static final int LANGUAGE_ID_MACINTOSH_KANNADA = 73;
	public static final int LANGUAGE_ID_MACINTOSH_TAMIL = 74;
	public static final int LANGUAGE_ID_MACINTOSH_TELUGU = 75;
	public static final int LANGUAGE_ID_MACINTOSH_SINHALESE = 76;
	public static final int LANGUAGE_ID_MACINTOSH_BURMESE = 77;
	public static final int LANGUAGE_ID_MACINTOSH_KHMER = 78;
	public static final int LANGUAGE_ID_MACINTOSH_LAO = 79;
	public static final int LANGUAGE_ID_MACINTOSH_VIETNAMESE = 80;
	public static final int LANGUAGE_ID_MACINTOSH_INDONESIAN = 81;
	public static final int LANGUAGE_ID_MACINTOSH_TAGALOG = 82;
	public static final int LANGUAGE_ID_MACINTOSH_MALAY_ROMAN = 83;
	public static final int LANGUAGE_ID_MACINTOSH_MALAY_ARABIC = 84;
	public static final int LANGUAGE_ID_MACINTOSH_AMHARIC = 85;
	public static final int LANGUAGE_ID_MACINTOSH_TIGRINYA = 86;
	public static final int LANGUAGE_ID_MACINTOSH_GALLA = 87;
	public static final int LANGUAGE_ID_MACINTOSH_SOMALI = 88;
	public static final int LANGUAGE_ID_MACINTOSH_SWAHILI = 89;
	public static final int LANGUAGE_ID_MACINTOSH_KINYARWANDA = 90;
	public static final int LANGUAGE_ID_MACINTOSH_RUNDI = 91;
	public static final int LANGUAGE_ID_MACINTOSH_NYANJA = 92;
	public static final int LANGUAGE_ID_MACINTOSH_MALAGASY = 93;
	public static final int LANGUAGE_ID_MACINTOSH_ESPERANTO = 94;
	public static final int LANGUAGE_ID_MACINTOSH_WELSH = 128;
	public static final int LANGUAGE_ID_MACINTOSH_BASQUE = 129;
	public static final int LANGUAGE_ID_MACINTOSH_CATALAN = 130;
	public static final int LANGUAGE_ID_MACINTOSH_LATIN = 131;
	public static final int LANGUAGE_ID_MACINTOSH_QUECHUA = 132;
	public static final int LANGUAGE_ID_MACINTOSH_GUARANI = 133;
	public static final int LANGUAGE_ID_MACINTOSH_AYMARA = 134;
	public static final int LANGUAGE_ID_MACINTOSH_TATAR = 135;
	public static final int LANGUAGE_ID_MACINTOSH_UIGHUR = 136;
	public static final int LANGUAGE_ID_MACINTOSH_DZONGKHA = 137;
	public static final int LANGUAGE_ID_MACINTOSH_JAVANESE_ROMAN = 138;
	public static final int LANGUAGE_ID_MACINTOSH_SUNDANESE_ROMAN = 139;
	public static final int LANGUAGE_ID_MACINTOSH_GALICIAN = 140;
	public static final int LANGUAGE_ID_MACINTOSH_AFRIKAANS = 141;
	public static final int LANGUAGE_ID_MACINTOSH_BRETON = 142;
	public static final int LANGUAGE_ID_MACINTOSH_INUKTITUT = 143;
	public static final int LANGUAGE_ID_MACINTOSH_SCOTTISH_GAELIC = 144;
	public static final int LANGUAGE_ID_MACINTOSH_MANX_GAELIC = 145;
	public static final int LANGUAGE_ID_MACINTOSH_IRISH_GAELIC_WITH_DOT_ABOVE = 146;
	public static final int LANGUAGE_ID_MACINTOSH_TONGAN = 147;
	public static final int LANGUAGE_ID_MACINTOSH_GREEK_POLYTONIC = 148;
	public static final int LANGUAGE_ID_MACINTOSH_GREENLANDIC = 149;
	public static final int LANGUAGE_ID_MACINTOSH_AZERBAIJANI_ROMAN = 150;
	
	public static final int LANGUAGE_ID_WINDOWS_ENGLISH = 1033;
}
